-------------------------------------------------------------------------------
--  Copyright (c) 2016 Eclektek LLC. All rights reserved.
--
--
--  Developed by: William Carter
--
--  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and  
--  associated documentation files (the "Software"), to deal with the Software without restriction,  
--  including without limitation the rights to use, copy, modify, merge, publish, sublicense and/or    
--  distribute copies of the Software, and to permit persons to whom the Software is furnished to do so,  
--  subject to the following conditions:
--
--      Redistributions of source code must retain the above copyright notice, this list of conditions 
--      and the following disclaimers.
--      Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
--      and the following disclaimers in the documentation and/or other materials provided with the distribution.
--      Neither the names of William Carter, Eclektek LLC, nor the names of its 
--      contributors may be used to endorse or promote products derived from this Software without 
--      specific prior written permission from Eclektek LLC.
--      This Software is provided free of charge and Eclektek LLC reserves the right to sell copies of the 
--      Software in it original form. Anyone sub-licensing this Software will provide it free of charge.
--
--  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
--  LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
--  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
--  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
--  SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
-------------------------------------------------------------------------------
--  History :                                                                       
--                                                                                                                  
--  Date      By    Comments                                         
--  --------  ----  ------------------------------------------------------------
--  12/07/16  W.C.  Created                                     
--
-------------------------------------------------------------------------------
--  Project       :                                                           
--  PCB           : 
--  Entity Name   : DEBUG_INF
--  File Name     : DEBUG_INF.vhd
--  Design Parent : none                                                          
--                                                                             
-------------------------------------------------------------------------------
--  Dependencies:
--
--  DEBUG_INF.vhd is uses the following components:
--
--  DEBUG_INF.vhd is uses the following packages:
--    Package        none
--
-------------------------------------------------------------------------------
--  DEBUG_INF.vhd
--
-------------------------------------------------------------------------------
--  Limitations
------------------------------------------------------------------------------
--  ToDo
--
-------------------------------------------------------------------------------
--  NOTES:
--
-------------------------------------------------------------------------------


Library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_unsigned.all;

library UNISIM;
  use UNISIM.vcomponents.all;
    
entity DEBUG_INF is
  port(
                                                                    
        clk_100                         : in    std_logic; 
        areset                          : in    std_logic;  
        
        -- General I/O resources
        clk100_heartbeat_led            : out   std_logic;
        clk_si570_heartbeat_led         : out   std_logic;
          
        -- Reg0 Control out                                            
        sysreset_in                     : out   std_logic;
        gtreset_in                      : out   std_logic;
        cdrovrden_in                    : out   std_logic;
        loopback_in                     : out   std_logic_vector(2 downto 0);
        quality_counter_rst_in          : out   std_logic;
        test_fsm_rst_in                 : out   std_logic;
        test_enable_rst_in              : out   std_logic;
        counter_sel_in                  : out   std_logic_vector(2 downto 0);
        test_fsm_check_time_sel_in      : out   std_logic_vector(2 downto 0);
                                        
        -- Reg4 Status in               
        hard_err                        : in    std_logic;
        soft_err                        : in    std_logic;
        channel_up                      : in    std_logic;
        lane_up                         : in    std_logic_vector(3 downto 0);
        data_err_count                  : in    std_logic_vector(7 downto 0);
        test_failed                     : in    std_logic;
        test_passed                     : in    std_logic;
                                        
        -- Reg5 Status in               
        chan_up_transcnt_20bit_i        : in    std_logic_vector(19 downto 0);
                                        
        -- Reg6 Status in               
        lnkrst_cnt_20bit_vio_i          : in    std_logic_vector(19 downto 0);
          
        -- FIFO interface
        fifo_rdclk                      :out   std_logic;
        fifo_rden                       :out   std_logic;
        fifo_rdata                      :in    std_logic_vector(47 downto 0);
        fifo_rst                        :out   std_logic;
        fifo_full_flg                   :in    std_logic;
        tcount                          :in    std_logic_vector(47 downto 0);
        
        -- GENESYS2 Fan
        xadc_fan_ctrl                   : out   std_logic;                      
        
        -- I2C PHYSICAL INTERFACE
        scl                             : out   std_logic; 
        sda_in                          : in    std_logic; 
        sda_out                         : out   std_logic; 
        sda_oe                          : out   std_logic; 
        
        -- UART
        uart_rx                         : out   std_logic;
        uart_tx                         : in    std_logic;
        
        -- FMC Si570 clock from IBUFDS
        vclk                            : in    std_logic
  );
end entity;

architecture rtl of DEBUG_INF is

-------------------------------------------------------------------------------
--  Component Declarations
-------------------------------------------------------------------------------
  
  component Genesys2_Si570_INF
    port(
          -- GLOBAL SIGNALS
          clk_100                         : in    std_logic; 
          areset                          : in    std_logic;  
          smreset                         : in    std_logic;
          
          -- General I/O resources
          clk100_heartbeat_led            : out   std_logic;
          clk_si570_heartbeat_led         : out   std_logic;
          si570_div16_mon                 : out   std_logic;
            
          -- UART Debug interface
          reg_address                     : out   std_logic_vector(3 downto 0);
          reg_data_out                    : out   std_logic_vector(47 downto 0);
          reg_write_stb                   : out   std_logic;
          reg_data_in                     : in    std_logic_vector(47 downto 0);
          reg_read_stb                    : out   std_logic;
          busy                            : out   std_logic;
          
          -- I2C PHYSICAL INTERFACE
          scl                             : out   std_logic; 
          sda_in                          : in    std_logic; 
          sda_out                         : out   std_logic; 
          sda_oe                          : out   std_logic; 
          
          -- UART
          uart_rx                         : out   std_logic;
          uart_tx                         : in    std_logic;
          
          -- FMC Si570 clock from IBUFDS
          vclk                            : in    std_logic
    );
  end component;


-------------------------------------------------------------------------------
--  Type Declarations
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------

constant XADC_38_C_FAN_ON_TEMP           : std_logic_vector(15 downto 0) := X"9E0B"; -- 38 degress C
constant XADC_40_C_FAN_ON_TEMP           : std_logic_vector(15 downto 0) := X"9F10"; -- 40 degress C
constant XADC_35_C_FAN_OFF_TEMP          : std_logic_vector(15 downto 0) := X"9C85"; -- 35 degress C
constant MAX_TEMP_SERVICE_COUNT          : std_logic_vector(27 downto 0) := X"5F5E100"; -- 100000000 for 1 second interval

-------------------------------------------------------------------------------
--  Signal Declarations
-------------------------------------------------------------------------------

signal ireg_address               : std_logic_vector(3 downto 0);
signal ireg_data_out              : std_logic_vector(47 downto 0);
signal ireg_write_stb             : std_logic;
signal ireg_data_in               : std_logic_vector(47 downto 0);
signal ireg_read_stb              : std_logic;
signal ibusy                      : std_logic;
signal dbg_reg0_r                 : std_logic_vector(47 downto 0);
signal dbg_reg1_r                 : std_logic_vector(47 downto 0);
signal dbg_reg2_r                 : std_logic_vector(47 downto 0);
signal dbg_reg3_r                 : std_logic_vector(47 downto 0);
signal dbg_reg4_r                 : std_logic_vector(47 downto 0);
signal dbg_reg5_r                 : std_logic_vector(47 downto 0);
signal dbg_reg6_r                 : std_logic_vector(47 downto 0);
signal dbg_reg7_r                 : std_logic_vector(47 downto 0);
signal dbg_reg8_r                 : std_logic_vector(47 downto 0);
signal dbg_reg9_r                 : std_logic_vector(47 downto 0);
signal dbg_reg10_r                : std_logic_vector(47 downto 0);
signal dbg_reg11_r                : std_logic_vector(47 downto 0);
signal reg11_wr_stb               : std_logic;
signal reg11_wr_data              : std_logic_vector(47 downto 0);
signal smreset                    : std_logic := '0';
signal xadc_di                    : std_logic_vector(15 downto 0);
signal xadc_do                    : std_logic_vector(15 downto 0);
signal xadc_dwe                   : std_logic;
signal xadc_drdy                  : std_logic;
signal xadc_den                   : std_logic;
signal xadc_addr                  : std_logic_vector(6 downto 0);
signal xadc_alarm                 : std_logic_vector(7 downto 0);
signal xadc_overtemp_alarm        : std_logic;
signal xadc_we_ctrl_bit           : std_logic;
signal fan_on                     : std_logic;
signal temp_srvc_count            : std_logic_vector(27 downto 0);
signal timer_wrap                 : std_logic;

-------------------------------------------------------------------------------
--  Procedure Declarations
-------------------------------------------------------------------------------
  
  
begin


-------------------------------------------------------------------------------
--  Component Instantiations
-------------------------------------------------------------------------------

  Si570_inst : Genesys2_Si570_INF 
  port map(
        clk_100                    => clk_100,
        areset                     => areset,
        smreset                    => smreset,
        clk100_heartbeat_led       => clk100_heartbeat_led,
        clk_si570_heartbeat_led    => clk_si570_heartbeat_led,
        si570_div16_mon            => open,
        reg_address                => ireg_address,
        reg_data_out               => ireg_data_out,
        reg_write_stb              => ireg_write_stb,
        reg_data_in                => ireg_data_in,
        reg_read_stb               => ireg_read_stb,
        busy                       => ibusy,
        scl                        => scl,
        sda_in                     => sda_in,
        sda_out                    => sda_out,
        sda_oe                     => sda_oe,
        uart_rx                    => uart_rx,
        uart_tx                    => uart_tx,
        vclk                       => vclk);


  -- XADC: Dual 12-Bit 1MSPS Analog-to-Digital Converter
  -- 7 Series
  -- Xilinx HDL Libraries Guide, version 13.4
  XADC_inst : XADC
  generic map (
    -- INIT_40 - INIT_42: XADC configuration registers
    INIT_40 => X"A000", -- 64 sample averaging enabled
    INIT_41 => X"2ef0",
    INIT_42 => X"0800", -- ADCCLK = 100/8 = 12.5 Mhz
    -- INIT_48 - INIT_4F: Sequence Registers
    INIT_48 => X"4701", -- CHSEL1 - enable Temp VCCINT, VCCAUX, VCCBRAM, and calibration
    INIT_49 => X"0000", -- CHSEL2 - enable nothing else
    INIT_4A => X"0001", -- SEQAVG1 disabled all channels
    INIT_4B => X"0000", -- SEQAVG2 disabled all channels
    INIT_4C => X"0000", -- SEQINMODE0 - all channels unipolar
    INIT_4D => X"0000", -- SEQINMODE1 - all channels unipolar
    INIT_4E => X"0000", -- SEQACQ0 - No extra settling time all channels
    INIT_4F => X"0000", -- SEQACQ1 - No extra settling time all channels
    -- INIT_50 - INIT_58, INIT5C: Alarm Limit Registers
    INIT_50 => X"b5ed", -- Temp upper alarm trigger 85C
    INIT_51 => X"5999", -- Vccint upper alarm limit 1.05V
    INIT_52 => X"A147", -- Vccaux upper alarm limit 1.89V
    INIT_53 => X"dddd", -- OT upper alarm limit 125C - see Thermal Management
    INIT_54 => X"a93a", -- Temp lower alarm reset 60C
    INIT_55 => X"5111", -- Vccint lower alarm limit 0.95V
    INIT_56 => X"91Eb", -- Vccaux lower alarm limit 1.71V
    INIT_57 => X"ae4e", -- OT lower alarm reset 70C - see Thermal Management
    INIT_58 => X"5999", -- VCCBRAM upper alarm limit 1.05V
    INIT_5C => X"5111", -- VCCBRAM lower alarm limit 0.95V
    -- Simulation attributes: Set for proepr simulation behavior
    SIM_DEVICE => "7SERIES", -- Select target device (values)
    SIM_MONITOR_FILE => "design.txt" -- Analog simulation data file name
  )
  port map (
    -- ALARMS: 8-bit (each) output: ALM, OT
    ALM             => xadc_alarm,          -- 8-bit output: Output alarm for temp, Vccint, Vccaux and Vccbram
    OT              => xadc_overtemp_alarm, -- 1-bit output: Over-Temperature alarm
    -- Dynamic Reconfiguration Port (DRP): 16-bit (each) output: Dynamic Reconfiguration Ports
    DO              => xadc_do,             -- 16-bit output: DRP output data bus
    DRDY            => xadc_drdy,           -- 1-bit output: DRP data ready
    -- STATUS: 1-bit (each) output: XADC status ports
    BUSY            => open, -- 1-bit output: ADC busy output
    CHANNEL         => open, -- 5-bit output: Channel selection outputs
    EOC             => open, -- 1-bit output: End of Conversion
    EOS             => open, -- 1-bit output: End of Sequence
    JTAGBUSY        => open, -- 1-bit output: JTAG DRP transaction in progress output
    JTAGLOCKED      => open, -- 1-bit output: JTAG requested DRP port lock
    JTAGMODIFIED    => open, -- 1-bit output: JTAG Write to the DRP has occurred
    MUXADDR         => open, -- 5-bit output: External MUX channel decode
    -- Auxiliary Analog-Input Pairs: 16-bit (each) input: VAUXP[15:0], VAUXN[15:0]
    VAUXN           => (others => '0'),     -- 16-bit input: N-side auxiliary analog input
    VAUXP           => (others => '0'),     -- 16-bit input: P-side auxiliary analog input
    -- CONTROL and CLOCK: 1-bit (each) input: Reset, conversion start and clock inputs
    CONVST          => '0',                 -- 1-bit input: Convert start input
    CONVSTCLK       => '0',                 -- 1-bit input: Convert start input
    RESET           => '0',                 -- 1-bit input: Active-high reset
    -- Dedicated Analog Input Pair: 1-bit (each) input: VP/VN
    VN              => '0',                 -- 1-bit input: N-side analog input
    VP              => '0',                 -- 1-bit input: P-side analog input
    -- Dynamic Reconfiguration Port (DRP): 7-bit (each) input: Dynamic Reconfiguration Ports
    DADDR           => xadc_addr,           -- 7-bit input: DRP address bus
    DCLK            => clk_100,             -- 1-bit input: DRP clock
    DEN             => xadc_den,            -- 1-bit input: DRP enable signal
    DI              => xadc_di,             -- 16-bit input: DRP input data bus
    DWE             => xadc_dwe             -- 1-bit input: DRP write enable
  );

-------------------------------------------------------------------------------
--  Concurrent Assignments
-------------------------------------------------------------------------------

   
  -- These are the debug registers available to this entity
  ireg_data_in  <= dbg_reg0_r     when ireg_address = X"0" else
                   dbg_reg1_r     when ireg_address = X"1" else
                   dbg_reg2_r     when ireg_address = X"2" else
                   dbg_reg3_r     when ireg_address = X"3" else
                   dbg_reg4_r     when ireg_address = X"4" else
                   dbg_reg5_r     when ireg_address = X"5" else
                   dbg_reg6_r     when ireg_address = X"6" else
                   dbg_reg7_r     when ireg_address = X"7" else
                   dbg_reg8_r     when ireg_address = X"8" else
                   dbg_reg9_r     when ireg_address = X"9" else
                   dbg_reg10_r    when ireg_address = X"A" else
                   dbg_reg11_r    when ireg_address = X"B" else
                   -- registers 12 through 15 are used by Si570_INF
                   (others => '0');

                     
-------------------------------------------------------------------------------
--  Process Declarations
-------------------------------------------------------------------------------


  -- Reg0 is the Control Register
  -- Reg8 is a read-back register for test purposes
  -- a process that allows the UART to write read-back registers
  UART_DBG_REG_WRITE_PROC : process(clk_100, areset)
  begin
    if areset = '1' then
      dbg_reg0_r      <= (others => '0');
      dbg_reg8_r      <= (others => '0');
    elsif rising_edge(clk_100) then
      if (ireg_write_stb = '1') then
        if ireg_address = X"0" then
          dbg_reg0_r   <= ireg_data_out;
        elsif ireg_address = X"8" then
          dbg_reg8_r   <= ireg_data_out;
        -- registers 12 through 15 are used by Si570_INF           
        end if;
      else
        dbg_reg0_r      <= dbg_reg0_r;
        dbg_reg8_r      <= dbg_reg8_r;
        -- registers 12 through 15 are used by Si570_INF  
      end if;
    end if;
  end process UART_DBG_REG_WRITE_PROC;
  
  -- Reg4 is a general purpose status register
  dbg_reg4_r    <= hard_err &
                   soft_err &
                   X"0000000" & "00" &
                   test_failed &
                   test_passed &
                   fifo_full_flg &
                   channel_up &      
                   lane_up &         
                   data_err_count;
  
  -- Reg5 provides access to the channel activation count
  dbg_reg5_r    <= X"0000000" & chan_up_transcnt_20bit_i;
  
  -- Reg6 provides access to the channel reset count
  dbg_reg6_r    <= X"0000000" & lnkrst_cnt_20bit_vio_i;
  
  -- Reg2 provides access to the aurora Rx data fifo
  dbg_reg2_r    <= fifo_rdata;
  
  -- Reg3 provides access to the fifo fill count
  dbg_reg3_r    <= tcount;
  
  
  -- R10 is the XADC Write Register. Writable by the UART
  xadc_we_ctrl_bit    <= '0'             when timer_wrap = '1' else dbg_reg10_r(47);
  xadc_addr           <= (others => '0') when timer_wrap = '1' else dbg_reg10_r(22 downto 16);
  xadc_di             <= (others => '0') when timer_wrap = '1' else dbg_reg10_r(15 downto 0);
  
  xadc_den            <= '1' when ireg_write_stb = '1' and ireg_address = X"A" else
                         '1' when timer_wrap = '1' else 
                         '0';
   
  -- In XADC DRP timing xadc_dwe and xadc_di is sampled on the rising_edge of DCLK following 
  -- assertion of xadc_den 
  process(clk_100,areset)
  begin
    if areset = '1' then
      xadc_dwe <= '0';
    elsif (rising_edge(clk_100)) then
      xadc_dwe <= xadc_den and xadc_we_ctrl_bit;
    end if;                                                                       
  end process ; 
  
  -- R11 XADC Read Register
  -- Writeable by the UART or this entity.The UART has write precedence.
  -- R11 is written here when the the XADC DRDY strobe is asserted
  -- R11   - XADC DRP DO output
  -- R11 can be cleared but not written by the UART 
  reg11_wr_data    <= xadc_overtemp_alarm & xadc_alarm & X"00000" & "000" & xadc_do;                 
  reg11_wr_stb     <= '1' when xadc_drdy = '1' else '0';  
  XADC_READ_REG11_WRITE_PROC : process(clk_100, areset)
  begin
    if areset = '1' then
      dbg_reg11_r      <= (others => '0');
    elsif rising_edge(clk_100) then
      if (ireg_write_stb = '1' and ireg_address = X"B") then
        dbg_reg11_r   <= (others => '0');
      elsif reg11_wr_stb = '1' then  
        dbg_reg11_r   <= reg11_wr_data;
      else
        dbg_reg11_r   <= dbg_reg11_r;
      end if;
    end if;
  end process XADC_READ_REG11_WRITE_PROC;

  
  -- xadc based fan control
  process(clk_100,areset)
  begin
    if areset = '1' then
      fan_on <= '0';
    elsif (rising_edge(clk_100)) then
      if dbg_reg11_r(15 downto 0) > XADC_38_C_FAN_ON_TEMP then
        fan_on <= '1';
      elsif dbg_reg11_r(15 downto 0) < XADC_35_C_FAN_OFF_TEMP then
        fan_on <= '0';
      else
        fan_on  <= fan_on;
      end if;
    end if;                                                                       
  end process ; 
  

  -- a process to measure out 1 second intervals 
  process(clk_100,areset)
  begin
    if areset = '1' then
      temp_srvc_count <= (others => '0');
      timer_wrap  <= '0';
    elsif (rising_edge(clk_100)) then
      if temp_srvc_count < MAX_TEMP_SERVICE_COUNT then
        temp_srvc_count <= temp_srvc_count + 1;
        timer_wrap  <= '0';
      else  
        temp_srvc_count <= (others => '0');
        timer_wrap  <= '1';
      end if;
    end if;                                                                       
  end process ; 


-------------------------------------------------------------------------------
--  Other Output Assignments
-------------------------------------------------------------------------------

  -- bitfield assignments to the Control register REG0
  sysreset_in                 <= dbg_reg0_r(47);               
  gtreset_in                  <= dbg_reg0_r(46);
  quality_counter_rst_in      <= dbg_reg0_r(45);
  test_fsm_rst_in             <= dbg_reg0_r(44);
  test_enable_rst_in          <= dbg_reg0_r(43);
  test_fsm_check_time_sel_in  <= dbg_reg0_r(9 downto 7);
  counter_sel_in              <= dbg_reg0_r(6 downto 4);
  cdrovrden_in                <= dbg_reg0_r(3);
  loopback_in                 <= dbg_reg0_r(2 downto 0);
  
  
  fifo_rdclk                  <= clk_100;
  -- get new fifo data after a read of Reg2 by the UART
  fifo_rden                   <= '1' when ireg_read_stb = '1' and ireg_address = X"2" else '0';
  fifo_rst                    <= dbg_reg0_r(12);
  
  
  
  -- xadc fan control for Genesys2 
  xadc_fan_ctrl               <= fan_on; 

end rtl;
