 ///////////////////////////////////////////////////////////////////////////////
 //
 // Project:  Aurora 64B/66B
 // Company:  Xilinx
 //
 //
 //
 // (c) Copyright 2008 - 2009 Xilinx, Inc. All rights reserved.
 //
 // This file contains confidential and proprietary information
 // of Xilinx, Inc. and is protected under U.S. and
 // international copyright and other intellectual property
 // laws.
 //
 // DISCLAIMER
 // This disclaimer is not a license and does not grant any
 // rights to the materials distributed herewith. Except as
 // otherwise provided in a valid license issued to you by
 // Xilinx, and to the maximum extent permitted by applicable
 // law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
 // WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
 // AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
 // BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
 // INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
 // (2) Xilinx shall not be liable (whether in contract or tort,
 // including negligence, or under any other theory of
 // liability) for any loss or damage of any kind or nature
 // related to, arising under or in connection with these
 // materials, including for any direct, or any indirect,
 // special, incidental, or consequential loss or damage
 // (including loss of data, profits, goodwill, or any type of
 // loss or damage suffered as a result of any action brought
 // by a third party) even if such damage or loss was
 // reasonably foreseeable or Xilinx had been advised of the
 // possibility of the same.
 //
 // CRITICAL APPLICATIONS
 // Xilinx products are not designed or intended to be fail-
 // safe, or for use in any application requiring fail-safe
 // performance, such as life-support or safety devices or
 // systems, Class III medical devices, nuclear facilities,
 // applications related to the deployment of airbags, or any
 // other applications that could lead to death, personal
 // injury, or severe property or environmental damage
 // (individually and collectively, "Critical
 // Applications"). Customer assumes the sole risk and
 // liability of any use of Xilinx products in Critical
 // Applications, subject only to applicable laws and
 // regulations governing limitations on product liability.
 //
 // THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
 // PART OF THIS FILE AT ALL TIMES.

 //
 ///////////////////////////////////////////////////////////////////////////////
 //
 //  EXAMPLE_DESIGN
 //
 //
 //  Description:  This module instantiates 4 lane Aurora Module.
 //                Used to exhibit functionality in hardware using the example design
 //                The User Interface is connected to Data Generator and Checker.
 ///////////////////////////////////////////////////////////////////////////////

 // aurora sample file, example design
 
 // Modified to replace the LABTOOLS with a UART based option

 `timescale 1 ns / 10 ps

   (* core_generation_info = "aurora_64b66b_0,aurora_64b66b_v11_1_1,{c_aurora_lanes=4,c_column_used=left,c_gt_clock_1=GTXQ3,c_gt_clock_2=None,c_gt_loc_1=X,c_gt_loc_10=X,c_gt_loc_11=X,c_gt_loc_12=X,c_gt_loc_13=2,c_gt_loc_14=4,c_gt_loc_15=1,c_gt_loc_16=3,c_gt_loc_17=X,c_gt_loc_18=X,c_gt_loc_19=X,c_gt_loc_2=X,c_gt_loc_20=X,c_gt_loc_21=X,c_gt_loc_22=X,c_gt_loc_23=X,c_gt_loc_24=X,c_gt_loc_25=X,c_gt_loc_26=X,c_gt_loc_27=X,c_gt_loc_28=X,c_gt_loc_29=X,c_gt_loc_3=X,c_gt_loc_30=X,c_gt_loc_31=X,c_gt_loc_32=X,c_gt_loc_33=X,c_gt_loc_34=X,c_gt_loc_35=X,c_gt_loc_36=X,c_gt_loc_37=X,c_gt_loc_38=X,c_gt_loc_39=X,c_gt_loc_4=X,c_gt_loc_40=X,c_gt_loc_41=X,c_gt_loc_42=X,c_gt_loc_43=X,c_gt_loc_44=X,c_gt_loc_45=X,c_gt_loc_46=X,c_gt_loc_47=X,c_gt_loc_48=X,c_gt_loc_5=X,c_gt_loc_6=X,c_gt_loc_7=X,c_gt_loc_8=X,c_gt_loc_9=X,c_lane_width=4,c_line_rate=2.7,c_gt_type=gtx,c_qpll=false,c_nfc=false,c_nfc_mode=IMM,c_refclk_frequency=135.0,c_simplex=false,c_simplex_mode=TX,c_stream=false,c_ufc=false,c_user_k=false,flow_mode=None,interface_mode=Framing,dataflow_config=Duplex}" *)
(* DowngradeIPIdentifiedWarnings="yes" *)
 module aurora_64b66b_0_exdes  #
 (
     parameter   USE_CORE_TRAFFIC     =   1,
     parameter   USR_CLK_PCOUNT     =  9'd200,
     parameter   EXAMPLE_SIMULATION =   0
      //pragma translate_off
        | 1
      //pragma translate_on
      ,
      parameter   USE_LABTOOLS       =   1
      //pragma translate_off
        & 0
      //pragma translate_on
 )
 (
     RESET,

     // Error Detection Interface
     HARD_ERR,
     SOFT_ERR,
     DATA_ERR_COUNT,
     // Status
     LANE_UP,
     CHANNEL_UP,

     // System Interface
     INIT_CLK_P,      
     INIT_CLK_N,
     PMA_INIT,
     
     //
     UART_RX_OUT,
     UART_TX_IN,
     FAN_PWM,
     FMC_SCL,
     FMC_SDA, 

     //70MHz DRP clk for Virtex-6 GTH
     DRP_CLK_IN_P,
     DRP_CLK_IN_N,
     // GTX Reference Clock Interface
     GTXQ3_P,
     GTXQ3_N,

     // GTX Serial I/O
     RXP,
     RXN,
     TXP,
     TXN

 );
 `define DLY #1


 //***********************************Port Declarations*******************************

       input               RESET;

     // Error Detection Interface
       output            HARD_ERR;
       output            SOFT_ERR;
       output [0:7]      DATA_ERR_COUNT;

     // Status
       output  [0:3]      LANE_UP;
       output             CHANNEL_UP;

       input              PMA_INIT;
       input              INIT_CLK_P;
       input              INIT_CLK_N;
       input              DRP_CLK_IN_P;
       input              DRP_CLK_IN_N;

     // GTX Reference Clock Interface
       input              GTXQ3_P;
       input              GTXQ3_N;


     // GTX Serial I/O
       input   [0:3]      RXP;
       input   [0:3]      RXN;

       output  [0:3]      TXP;
       output  [0:3]      TXN;

       output UART_RX_OUT;
       input  UART_TX_IN;
       output FAN_PWM;
       output FMC_SCL;
       inout  FMC_SDA; 
       

 //************************External Register Declarations*****************************

     //Error reporting signals
       reg                  HARD_ERR;
       reg                  SOFT_ERR;
(* KEEP = "TRUE" *)       reg       [0:7]      DATA_ERR_COUNT;

     //Global signals
       reg       [0:3]      LANE_UP;
       reg                  CHANNEL_UP;

 //********************************Wire Declarations**********************************

     wire    [280:0]          tied_to_ground_vec_i;
     wire            INIT_CLK_IN;

     //Dut1
     //TX Interface
 (* mark_debug = "true" *)        wire      [0:255]    tx_tdata_i; 
 (* mark_debug = "true" *)        wire                 tx_tvalid_i;
 (* mark_debug = "true" *)        wire      [0:31]      tx_tkeep_i;  
 (* mark_debug = "true" *)        wire                 tx_tlast_i;
 (* mark_debug = "true" *)        wire                 tx_tready_i;

     //RX Interface
 (* mark_debug = "true" *)        wire      [0:255]     rx_tdata_i;  
 (* mark_debug = "true" *)        wire                 rx_tvalid_i;
 (* mark_debug = "true" *)        wire      [0:31]      rx_tkeep_i;  
 (* mark_debug = "true" *)        wire                 rx_tlast_i;





     //TX Interface
       wire      [0:255]    tx_d_i;
       wire                 tx_src_rdy_n_i;
       wire      [0:4]       tx_rem_i;
       wire                 tx_sof_n_i;
       wire                 tx_eof_n_i;
       wire                 tx_dst_rdy_n_i;

     //RX Interface
       wire      [0:255]     rx_d_i;
       wire                 rx_src_rdy_n_i;
       wire      [0:4]       rx_rem_i;
       wire                 rx_sof_n_i;
       wire                 rx_eof_n_i;




     //Error Detection Interface
 (* mark_debug = "true" *)        wire                 hard_err_i;
 (* mark_debug = "true" *)        wire                 soft_err_i;

     //Status
 (* mark_debug = "true" *)        wire                 channel_up_i;
 (* mark_debug = "true" *)        wire      [0:3]      lane_up_i;
(* mark_debug = "true" *)       wire                 lane_up_vio_usrclk;
(* mark_debug = "true" *)       wire                 lane_up_vio_i;


     //System Interface
       wire                 reset_i ;
       wire                 gt_rxcdrovrden_i ;
       wire                 power_down_i;
       wire      [2:0]      loopback_i ;
(* mark_debug = "true" *)         wire                 gt_pll_lock_i ;
(* mark_debug = "true" *)         wire                 fsm_resetdone_i ;
       wire                 tx_out_clk_i ;

     // Error signals from the frame checker
 (* mark_debug = "true" *)        wire      [0:7]       data_err_count_o;
 (* mark_debug = "true" *)      wire                  data_err_init_clk_i;


     // clock
       wire               user_clk_i;
       wire               sync_clk_i;
  (* KEEP = "TRUE" *)  wire               INIT_CLK_i  /* synthesis syn_keep = 1 */;

     wire               drp_clk_i = INIT_CLK_i;
     wire               DRP_CLK_i;
     wire    [8:0] drpaddr_in_i;
     wire    [15:0]     drpdi_in_i;
       wire    [15:0]     drpdo_out_i;
       wire               drprdy_out_i;
       wire               drpen_in_i;
       wire               drpwe_in_i;
     wire    [8:0] drpaddr_in_lane1_i;
     wire    [15:0]     drpdi_in_lane1_i;
       wire    [15:0]     drpdo_out_lane1_i;
       wire               drprdy_out_lane1_i;
       wire               drpen_in_lane1_i;
       wire               drpwe_in_lane1_i;
     wire    [8:0] drpaddr_in_lane2_i;
     wire    [15:0]     drpdi_in_lane2_i;
       wire    [15:0]     drpdo_out_lane2_i;
       wire               drprdy_out_lane2_i;
       wire               drpen_in_lane2_i;
       wire               drpwe_in_lane2_i;
     wire    [8:0] drpaddr_in_lane3_i;
     wire    [15:0]     drpdi_in_lane3_i;
       wire    [15:0]     drpdo_out_lane3_i;
       wire               drprdy_out_lane3_i;
       wire               drpen_in_lane3_i;
       wire               drpwe_in_lane3_i;
       wire               link_reset_i;
       wire               sysreset_from_vio_i;
       wire               gtreset_from_vio_i;
       wire               rx_cdrovrden_i;
       wire               gt_reset_i;
       wire               gt_reset_i_tmp;
       wire               gt_reset_i_tmp2;
       wire               sysreset_from_vio_r3;
       wire               sysreset_from_vio_r3_initclkdomain;
       wire               gtreset_from_vio_r3;
       wire               tied_to_ground_i;
       wire               tied_to_vcc_i;
       wire               gt_reset_i_eff;
       wire               system_reset_i;
       wire                          pll_not_locked_i;
 
reg  pma_init_from_fsm = 0;
reg pma_init_from_fsm_r1 = 0;
reg lane_up_vio_usrclk_r1 = 0;
reg data_err_count_o_r1  = 0;
(* mark_debug = "TRUE" *) reg rx_tvalid_r = 1'd0;
(* mark_debug = "TRUE" *) wire rx_tvalid_r_i;
 
(* mark_debug = "TRUE" *) reg [8:0] usr_clk_counter = 9'd0;
(* mark_debug = "TRUE" *) wire usr_clk_count_done;
(* mark_debug = "TRUE" *) wire usr_clk_count_done_i;
 

     wire reset2FrameGenCheck;
 

 //*********************************Main Body of Code**********************************

     assign reset2FrameGenCheck = reset_i | !channel_up_i;
 




//--- Instance of GT differential buffer ---------//

     //____________________________Register User I/O___________________________________

     // Register User Outputs from core.
     always @(posedge user_clk_i)
     begin
         HARD_ERR         <=  hard_err_i;
         SOFT_ERR         <=  soft_err_i;
         LANE_UP          <=  lane_up_i;
         CHANNEL_UP       <=  channel_up_i;
         DATA_ERR_COUNT   <=  data_err_count_o;
     end

//   IBUFGDS drpclk_bufg_i
//     (
//        .I  (DRP_CLK_IN_P),
//        .IB  (DRP_CLK_IN_N),
//        .O  (DRP_CLK_i)
//     );

     //____________________________Register User I/O___________________________________

     // System Interface
     assign power_down_i      =   1'b0;
     assign tied_to_ground_i   =   1'b0;
     assign tied_to_ground_vec_i = 281'd0;
     assign tied_to_vcc_i      =   1'b1;
    // Native DRP Interface
     assign  drpaddr_in_i                     =  'h0;
     assign  drpdi_in_i                       =  16'h0;
     assign  drpwe_in_i     =  1'b0;
     assign  drpen_in_i     =  1'b0;
     assign  drpaddr_in_lane1_i                     =  'h0;
     assign  drpdi_in_lane1_i                       =  16'h0;
     assign  drpwe_in_lane1_i     =  1'b0;
     assign  drpen_in_lane1_i     =  1'b0;
     assign  drpaddr_in_lane2_i                     =  'h0;
     assign  drpdi_in_lane2_i                       =  16'h0;
     assign  drpwe_in_lane2_i     =  1'b0;
     assign  drpen_in_lane2_i     =  1'b0;
     assign  drpaddr_in_lane3_i                     =  'h0;
     assign  drpdi_in_lane3_i                       =  16'h0;
     assign  drpwe_in_lane3_i     =  1'b0;
     assign  drpen_in_lane3_i     =  1'b0;


// DRP clock

// Additions  

  wire  is_locked;
  wire  mmcm_rst;
  wire  clk100;
  
  assign mmcm_rst = 1'b0;
  assign DRP_CLK_i = clk100;
 
    
clk_wiz_0 drp_clk_inst
 (
 // Clock in ports
  .clk_in1_p(DRP_CLK_IN_P),    // 156.25 MHz
  .clk_in1_n(DRP_CLK_IN_N),    // 156.25 MHz
  // Clock out ports
  .clk_out1(clk100),     // 100 MHz
  // Status and control signals
  .reset(mmcm_rst), // input reset
  .locked(is_locked));      // output locked
     
  
   wire areset;
   wire smreset;
   wire clk100_heartbeat_led; 
   wire clk_si570_heartbeat_led;
   wire si570_div16_mon;
   wire [3:0] reg_address;
   wire [47:0] reg_data_out;
   wire reg_write_stb;
   reg [47:0] reg_data_in;
   wire reg_read_stb;
   wire busy;      
   wire scl;  
//   wire sda_in;
   wire sda_out;
   wire sda_oe;
//   wire uart_rx;
//   wire uart_tx;
//   wire vclk_i;

   wire fifo_rden;
   wire fifo_rdclk;
   wire [63:0] fifo_rdata;
   wire fifo_rst;
   wire fifo_full_flg;
   reg [47:0] tcount;
 
   assign areset = RESET;
   assign smreset = RESET; // TO BE CHANGED TO A UART CONTROL BIT
   
       
// I needed a VHDL wrapper in order to use the GENESYS2_Si570_INF.ngc IP as the IP was
// created with VHDL and I haven't found a way to instantiate it in Verilog directly....
  DEBUG_INF  
  (
          // GLOBAL SIGNALS
          .clk_100                    (clk100),                     // in    std_logic; 
          .areset                     (areset),                     // in    std_logic;  
                                                                    
          // General I/O resources                                  
          .clk100_heartbeat_led       (clk100_heartbeat_led),       // out   std_logic;
          .clk_si570_heartbeat_led    (clk_si570_heartbeat_led),    // out   std_logic;
                                                                    
          // Reg0 Control out                                            
          .sysreset_in                (sysreset_in),                // out   std_logic;
          .gtreset_in                 (gtreset_in),                 // out   std_logic;
          .cdrovrden_in               (cdrovrden_in),               // out   std_logic;
          .loopback_in                (loopback_in),                // out   std_logic_vector(2 downto 0);
          .quality_counter_rst_in     (quality_counter_rst_in),     // out   std_logic;
          .test_fsm_rst_in            (test_fsm_rst_in),            // out   std_logic;
          .test_enable_rst_in         (test_enable_rst_in),         // out   std_logic;
          .counter_sel_in             (counter_sel_in),             // out   std_logic_vector(2 downto 0);
          .test_fsm_check_time_sel_in (test_fsm_check_time_sel_in), // out   std_logic_vector(2 downto 0);
                   
          // Reg4 Status in
          .hard_err                   (hard_err_i),                 // in    std_logic;
          .soft_err                   (soft_err_i),                 // in    std_logic;
          .channel_up                 (channel_up_i),               // in    std_logic;
          .lane_up                    (lane_up_i),                  // in    std_logic_vector(3 downto 0);
          .data_err_count             (data_err_count_o),           // in    std_logic_vector(7 downto 0);
          .test_failed                (test_failed_r),              // in    std_logic;
          .test_passed                (test_passed_r),              // in    std_logic;
          
          // Reg5 Status in
          .chan_up_transcnt_20bit_i   (chan_up_transcnt_20bit_i),   // in    std_logic_vector(19 downto 0);
          
          // Reg6 Status in
          .lnkrst_cnt_20bit_vio_i     (lnkrst_cnt_20bit_vio_i),     // in    std_logic_vector(19 downto 0);
          
          // FIFO interface
          .fifo_rdclk                 (fifo_rdclk),                 // out   std_logic;
          .fifo_rden                  (fifo_rden),                  // out   std_logic;
          .fifo_rdata                 (fifo_rdata[47:0]),           // in    std_logic_vector(47 downto 0);
          .fifo_rst                   (fifo_rst),                   // out   std_logic;
          .fifo_full_flg              (fifo_full_flg),              // in    std_logic;
          .tcount                     (tcount),                     // in    std_logic_vector(47 downto 0);

        
          // GENESYS2 Fan
          .xadc_fan_ctrl              (FAN_PWM),                      
                                    
          // I2C PHYSICAL INTERFACE       
          .scl                        (scl),                        // out   std_logic; 
          .sda_in                     (FMC_SDA),                    // in    std_logic; 
          .sda_out                    (sda_out),                    // out   std_logic; 
          .sda_oe                     (sda_oe),                     // out   std_logic; 
                                                                    
          // UART                                                   
          .uart_rx                    (UART_RX_OUT),                // out   std_logic;
          .uart_tx                    (UART_TX_IN),                 // in    std_logic;
                                                                    
          // FMC Si570 clock from IBUFDS                            
          .vclk                       (DRP_CLK_i)                   // in    std_logic
    );
  
  
       
  assign FMC_SDA = sda_out ? sda_oe : 1'bz; 
  
  reg fifo_wren;
  wire fifo_empty_flg;

  always @(posedge user_clk_i)
  begin
    if (fifo_rst == 1'b0 && fifo_full_flg == 1'b0 && rx_tvalid_i == 1'b1)
      fifo_wren <= 1'b1;
    else
      fifo_wren <= 1'b0;
  end
  
  
// capture the high 8 bits of 128K Rx words ( 256 bits )  
fifo_generator_0 data_capture_fifo (
  .rst(fifo_rst),         // input wire rst
  .wr_clk(user_clk_i),    // input wire wr_clk
  .rd_clk(fifo_rdclk),    // input wire rd_clk
  .din(rx_tdata_i[0:7]),  // input wire [7 : 0] din
  .wr_en(fifo_wren),      // input wire wr_en
  .rd_en(fifo_rden),      // input wire rd_en
  .dout(fifo_rdata),      // output wire [63 : 0] dout
  .full(fifo_full_flg),   // output wire full
  .empty(fifo_empty_flg)  // output wire empty
);
  
// measure the time to fill the 128K deep FIFO
always @(posedge user_clk_i)
begin
  if (fifo_rst)
    tcount <= 48'h000000000000;
  else
    if (tcount < 48'hFFFFFFFFFFFF  && fifo_full_flg == 1'b0)
    tcount <= tcount + 1;
      
end



// Additions  
          

//reset syncronization
wire sysreset_from_vio_i_eff = sysreset_from_vio_i | gt_reset_i_tmp;

aurora_64b66b_0_rst_sync_exdes   u_rst_sync_sysrst
     (
       .prmry_in     (sysreset_from_vio_i_eff),
       .scndry_aclk  (user_clk_i),
       .scndry_out   (sysreset_from_vio_r3)
      );

aurora_64b66b_0_rst_sync_exdes   u_rst_sync_sysrst_initclk
     (
       .prmry_in     (sysreset_from_vio_i_eff),
       .scndry_aclk  (INIT_CLK_i),
       .scndry_out   (sysreset_from_vio_r3_initclkdomain)
      );

aurora_64b66b_0_rst_sync_exdes   u_rst_sync_gtreset
     (
       .prmry_in     (gtreset_from_vio_i),
       .scndry_aclk  (INIT_CLK_i),
       .scndry_out   (gtreset_from_vio_r3)
      );

always @(posedge INIT_CLK_i)
begin
    pma_init_from_fsm_r1 <= `DLY  pma_init_from_fsm;
end



   reg [127:0]        pma_init_stage = 128'h0;
 (* mark_debug = "true" *)  reg [23:0]         pma_init_pulse_width_cnt;
   reg pma_init_assertion = 1'b0;
 (* mark_debug = "true" *)  reg pma_init_assertion_r;
   reg gt_reset_i_delayed_r1;
 (* mark_debug = "true" *)   reg gt_reset_i_delayed_r2;
   wire gt_reset_i_delayed;



     generate
        always @(posedge INIT_CLK_i)
        begin
            pma_init_stage[127:0] <= {pma_init_stage[126:0], gt_reset_i_tmp};
        end

        assign gt_reset_i_delayed = pma_init_stage[127];

        always @(posedge INIT_CLK_i)
        begin
            gt_reset_i_delayed_r1     <=  gt_reset_i_delayed;
            gt_reset_i_delayed_r2     <=  gt_reset_i_delayed_r1;
            pma_init_assertion_r  <= pma_init_assertion;
            if(~gt_reset_i_delayed_r2 & gt_reset_i_delayed_r1 & ~pma_init_assertion & (pma_init_pulse_width_cnt != 24'hFFFFFF))
                pma_init_assertion <= 1'b1;
            else if (pma_init_assertion & pma_init_pulse_width_cnt == 24'hFFFFFF)
                pma_init_assertion <= 1'b0;

            if(pma_init_assertion)
                pma_init_pulse_width_cnt <= pma_init_pulse_width_cnt + 24'h1;
        end



    if(EXAMPLE_SIMULATION)
    assign gt_reset_i_eff = gt_reset_i_delayed;
    else
    assign gt_reset_i_eff = pma_init_assertion ? 1'b1 : gt_reset_i_delayed;


     if(USE_LABTOOLS)
     begin:chip_reset
     assign  gt_reset_i_tmp = PMA_INIT | gtreset_from_vio_r3 | pma_init_from_fsm_r1;
     assign  reset_i  =  RESET | sysreset_from_vio_r3;
     assign  gt_reset_i = gt_reset_i_eff;
     assign  gt_rxcdrovrden_i  =  rx_cdrovrden_i;
     end
     else
     begin:no_chip_reset
     assign  gt_reset_i_tmp = PMA_INIT;
     assign  reset_i  =   RESET | gt_reset_i_tmp2;
     assign  gt_reset_i = gt_reset_i_eff;
     assign  gt_rxcdrovrden_i  =  1'b0;
     assign  loopback_i  =  3'b000;
     end

     if(!USE_LABTOOLS)
     begin
aurora_64b66b_0_rst_sync_exdes   u_rst_sync_gtrsttmpi
     (
       .prmry_in     (gt_reset_i_tmp),
       .scndry_aclk  (user_clk_i),
       .scndry_out   (gt_reset_i_tmp2)
      );
     end

     endgenerate

     //___________________________Module Instantiations_________________________________

// this is non shared mode, the clock, GT common are part of example design.
    aurora_64b66b_0_support
aurora_64b66b_0_block_i
     (
        // TX AXI4-S Interface
         .s_axi_tx_tdata(tx_tdata_i),
         .s_axi_tx_tlast(tx_tlast_i),
         .s_axi_tx_tkeep(tx_tkeep_i),
         .s_axi_tx_tvalid(tx_tvalid_i),
         .s_axi_tx_tready(tx_tready_i),


        // RX AXI4-S Interface
         .m_axi_rx_tdata(rx_tdata_i),
         .m_axi_rx_tlast(rx_tlast_i),
         .m_axi_rx_tkeep(rx_tkeep_i),
         .m_axi_rx_tvalid(rx_tvalid_i),



        // GT Serial I/O
         .rxp(RXP),
         .rxn(RXN),

         .txp(TXP),
         .txn(TXN),


        //GT Reference Clock Interface
        .gt_refclk1_p (GTXQ3_P),
        .gt_refclk1_n (GTXQ3_N),
        // Error Detection Interface
         .hard_err              (hard_err_i),
         .soft_err              (soft_err_i),

        // Status
         .channel_up            (channel_up_i),
         .lane_up               (lane_up_i),

        // System Interface
         .init_clk_out          (INIT_CLK_i),
         .user_clk_out          (user_clk_i),

         .sync_clk_out(sync_clk_i),
         .reset_pb(reset_i),
         .gt_rxcdrovrden_in(gt_rxcdrovrden_i),
         .power_down(power_down_i),
         .loopback(loopback_i),
         .pma_init(gt_reset_i),
         .gt_pll_lock(gt_pll_lock_i),
         .drp_clk_in(DRP_CLK_i),// (drp_clk_i),
    //---------------------- GT DRP Ports ----------------------
         .drpaddr_in          (drpaddr_in_i),
         .drpdi_in            (drpdi_in_i),
         .drpdo_out           (drpdo_out_i),
         .drprdy_out          (drprdy_out_i),
         .drpen_in            (drpen_in_i),
         .drpwe_in            (drpwe_in_i),
         .drpaddr_in_lane1          (drpaddr_in_lane1_i),
         .drpdi_in_lane1            (drpdi_in_lane1_i),
         .drpdo_out_lane1           (drpdo_out_lane1_i),
         .drprdy_out_lane1          (drprdy_out_lane1_i),
         .drpen_in_lane1            (drpen_in_lane1_i),
         .drpwe_in_lane1            (drpwe_in_lane1_i),
         .drpaddr_in_lane2          (drpaddr_in_lane2_i),
         .drpdi_in_lane2            (drpdi_in_lane2_i),
         .drpdo_out_lane2           (drpdo_out_lane2_i),
         .drprdy_out_lane2          (drprdy_out_lane2_i),
         .drpen_in_lane2            (drpen_in_lane2_i),
         .drpwe_in_lane2            (drpwe_in_lane2_i),
         .drpaddr_in_lane3          (drpaddr_in_lane3_i),
         .drpdi_in_lane3            (drpdi_in_lane3_i),
         .drpdo_out_lane3           (drpdo_out_lane3_i),
         .drprdy_out_lane3          (drprdy_out_lane3_i),
         .drpen_in_lane3            (drpen_in_lane3_i),
         .drpwe_in_lane3            (drpwe_in_lane3_i),


         .init_clk_p                            (INIT_CLK_P),
         .init_clk_n                            (INIT_CLK_N),
         .link_reset_out                        (link_reset_i),
         .mmcm_not_locked_out                   (pll_not_locked_i),

         .gt_rxusrclk_out (),
         //-------------------- gt0 debug ports start ---------------------------
         //------------------------ RX Margin Analysis Ports ------------------------
         .gt0_eyescandataerror_out         (),
         .gt0_eyescanreset_in              (tied_to_ground_i),
         .gt0_eyescantrigger_in            (tied_to_ground_i),
         //------------------- Receive Ports - RX Equalizer Ports -------------------
         .gt0_rxcdrhold_in                 (tied_to_ground_i),
         .gt0_rxlpmhfovrden_in             (tied_to_ground_i),
         .gt0_rxdfeagchold_in              (tied_to_ground_i),
         .gt0_rxdfeagcovrden_in            (tied_to_ground_i),
         .gt0_rxdfelfhold_in               (tied_to_ground_i),
         .gt0_rxdfelpmreset_in             (tied_to_ground_i),
         .gt0_rxlpmlfklovrden_in           (tied_to_ground_i),
         .gt0_rxmonitorout_out             (),
         .gt0_rxmonitorsel_in              (2'b00),
         .gt0_rxlpmen_in                   (tied_to_ground_i),
         .gt0_rxpmareset_in                (tied_to_ground_i),
         .gt0_rxpcsreset_in                (tied_to_ground_i),
         .gt0_rxbufreset_in                (tied_to_ground_i),
         .gt0_rxprbssel_in                  (tied_to_ground_vec_i[2:0]    ),
         .gt0_rxprbserr_out                 (),
         .gt0_rxprbscntreset_in             (tied_to_ground_i),
         //---------------------- TX Configurable Driver Ports ----------------------
         .gt0_txpostcursor_in               (5'b00000),
         .gt0_txdiffctrl_in                 (4'b1000),
         .gt0_txmaincursor_in               (7'b0000000),
         .gt0_txprecursor_in                (5'b00000),
         //--------------- Transmit Ports - TX Polarity Control Ports ---------------
         .gt0_txpolarity_in                 (tied_to_ground_i),
         .gt0_txinhibit_in                  (tied_to_ground_i),
         .gt0_txpmareset_in                 (tied_to_ground_i),
         .gt0_txpcsreset_in                 (tied_to_ground_i),
         .gt0_txprbssel_in                  (tied_to_ground_vec_i[2:0]  ),
         .gt0_txprbsforceerr_in             (tied_to_ground_i),
         .gt0_txbufstatus_out               (),
         .gt0_dmonitorout_out               (),
         .gt0_txresetdone_out               (),
         .gt0_rxresetdone_out               (),
         .gt0_rxbufstatus_out               (),
         .gt0_cplllock_out                  (),
         //-------------------- gt1 debug ports start ---------------------------
         //------------------------ RX Margin Analysis Ports ------------------------
         .gt1_eyescandataerror_out         (),
         .gt1_eyescanreset_in              (tied_to_ground_i),
         .gt1_eyescantrigger_in            (tied_to_ground_i),
         //------------------- Receive Ports - RX Equalizer Ports -------------------
         .gt1_rxcdrhold_in                 (tied_to_ground_i),
         .gt1_rxlpmhfovrden_in             (tied_to_ground_i),
         .gt1_rxdfeagchold_in              (tied_to_ground_i),
         .gt1_rxdfeagcovrden_in            (tied_to_ground_i),
         .gt1_rxdfelfhold_in               (tied_to_ground_i),
         .gt1_rxdfelpmreset_in             (tied_to_ground_i),
         .gt1_rxlpmlfklovrden_in           (tied_to_ground_i),
         .gt1_rxmonitorout_out             (),
         .gt1_rxmonitorsel_in              (2'b00),
         .gt1_rxlpmen_in                   (tied_to_ground_i),
         .gt1_rxpmareset_in                (tied_to_ground_i),
         .gt1_rxpcsreset_in                (tied_to_ground_i),
         .gt1_rxbufreset_in                (tied_to_ground_i),
         .gt1_rxprbssel_in                  (tied_to_ground_vec_i[2:0]    ),
         .gt1_rxprbserr_out                 (),
         .gt1_rxprbscntreset_in             (tied_to_ground_i),
         //---------------------- TX Configurable Driver Ports ----------------------
         .gt1_txpostcursor_in               (5'b00000),
         .gt1_txdiffctrl_in                 (4'b1000),
         .gt1_txmaincursor_in               (7'b0000000),
         .gt1_txprecursor_in                (5'b00000),
         //--------------- Transmit Ports - TX Polarity Control Ports ---------------
         .gt1_txpolarity_in                 (tied_to_ground_i),
         .gt1_txinhibit_in                  (tied_to_ground_i),
         .gt1_txpmareset_in                 (tied_to_ground_i),
         .gt1_txpcsreset_in                 (tied_to_ground_i),
         .gt1_txprbssel_in                  (tied_to_ground_vec_i[2:0]  ),
         .gt1_txprbsforceerr_in             (tied_to_ground_i),
         .gt1_txbufstatus_out               (),
         .gt1_dmonitorout_out               (),
         .gt1_txresetdone_out               (),
         .gt1_rxresetdone_out               (),
         .gt1_rxbufstatus_out               (),
         .gt1_cplllock_out                  (),
         //-------------------- gt2 debug ports start ---------------------------
         //------------------------ RX Margin Analysis Ports ------------------------
         .gt2_eyescandataerror_out         (),
         .gt2_eyescanreset_in              (tied_to_ground_i),
         .gt2_eyescantrigger_in            (tied_to_ground_i),
         //------------------- Receive Ports - RX Equalizer Ports -------------------
         .gt2_rxcdrhold_in                 (tied_to_ground_i),
         .gt2_rxlpmhfovrden_in             (tied_to_ground_i),
         .gt2_rxdfeagchold_in              (tied_to_ground_i),
         .gt2_rxdfeagcovrden_in            (tied_to_ground_i),
         .gt2_rxdfelfhold_in               (tied_to_ground_i),
         .gt2_rxdfelpmreset_in             (tied_to_ground_i),
         .gt2_rxlpmlfklovrden_in           (tied_to_ground_i),
         .gt2_rxmonitorout_out             (),
         .gt2_rxmonitorsel_in              (2'b00),
         .gt2_rxlpmen_in                   (tied_to_ground_i),
         .gt2_rxpmareset_in                (tied_to_ground_i),
         .gt2_rxpcsreset_in                (tied_to_ground_i),
         .gt2_rxbufreset_in                (tied_to_ground_i),
         .gt2_rxprbssel_in                  (tied_to_ground_vec_i[2:0]    ),
         .gt2_rxprbserr_out                 (),
         .gt2_rxprbscntreset_in             (tied_to_ground_i),
         //---------------------- TX Configurable Driver Ports ----------------------
         .gt2_txpostcursor_in               (5'b00000),
         .gt2_txdiffctrl_in                 (4'b1000),
         .gt2_txmaincursor_in               (7'b0000000),
         .gt2_txprecursor_in                (5'b00000),
         //--------------- Transmit Ports - TX Polarity Control Ports ---------------
         .gt2_txpolarity_in                 (tied_to_ground_i),
         .gt2_txinhibit_in                  (tied_to_ground_i),
         .gt2_txpmareset_in                 (tied_to_ground_i),
         .gt2_txpcsreset_in                 (tied_to_ground_i),
         .gt2_txprbssel_in                  (tied_to_ground_vec_i[2:0]  ),
         .gt2_txprbsforceerr_in             (tied_to_ground_i),
         .gt2_txbufstatus_out               (),
         .gt2_dmonitorout_out               (),
         .gt2_txresetdone_out               (),
         .gt2_rxresetdone_out               (),
         .gt2_rxbufstatus_out               (),
         .gt2_cplllock_out                  (),
         //-------------------- gt3 debug ports start ---------------------------
         //------------------------ RX Margin Analysis Ports ------------------------
         .gt3_eyescandataerror_out         (),
         .gt3_eyescanreset_in              (tied_to_ground_i),
         .gt3_eyescantrigger_in            (tied_to_ground_i),
         //------------------- Receive Ports - RX Equalizer Ports -------------------
         .gt3_rxcdrhold_in                 (tied_to_ground_i),
         .gt3_rxlpmhfovrden_in             (tied_to_ground_i),
         .gt3_rxdfeagchold_in              (tied_to_ground_i),
         .gt3_rxdfeagcovrden_in            (tied_to_ground_i),
         .gt3_rxdfelfhold_in               (tied_to_ground_i),
         .gt3_rxdfelpmreset_in             (tied_to_ground_i),
         .gt3_rxlpmlfklovrden_in           (tied_to_ground_i),
         .gt3_rxmonitorout_out             (),
         .gt3_rxmonitorsel_in              (2'b00),
         .gt3_rxlpmen_in                   (tied_to_ground_i),
         .gt3_rxpmareset_in                (tied_to_ground_i),
         .gt3_rxpcsreset_in                (tied_to_ground_i),
         .gt3_rxbufreset_in                (tied_to_ground_i),
         .gt3_rxprbssel_in                  (tied_to_ground_vec_i[2:0]    ),
         .gt3_rxprbserr_out                 (),
         .gt3_rxprbscntreset_in             (tied_to_ground_i),
         //---------------------- TX Configurable Driver Ports ----------------------
         .gt3_txpostcursor_in               (5'b00000),
         .gt3_txdiffctrl_in                 (4'b1000),
         .gt3_txmaincursor_in               (7'b0000000),
         .gt3_txprecursor_in                (5'b00000),
         //--------------- Transmit Ports - TX Polarity Control Ports ---------------
         .gt3_txpolarity_in                 (tied_to_ground_i),
         .gt3_txinhibit_in                  (tied_to_ground_i),
         .gt3_txpmareset_in                 (tied_to_ground_i),
         .gt3_txpcsreset_in                 (tied_to_ground_i),
         .gt3_txprbssel_in                  (tied_to_ground_vec_i[2:0]  ),
         .gt3_txprbsforceerr_in             (tied_to_ground_i),
         .gt3_txbufstatus_out               (),
         .gt3_dmonitorout_out               (),
         .gt3_txresetdone_out               (),
         .gt3_rxresetdone_out               (),
         .gt3_rxbufstatus_out               (),
         .gt3_cplllock_out                  (),
         //-------------------- gt4 debug ports end ---------------------------
         .gt_qplllock            (),



         .sys_reset_out                            (system_reset_i),
         .tx_out_clk                               (tx_out_clk_i)
     );

generate
 if (USE_CORE_TRAFFIC==1)
 begin : core_traffic

     //_____________________________ TX AXI SHIM _______________________________
aurora_64b66b_0_EXAMPLE_LL_TO_AXI #
     (
        .DATA_WIDTH(256),
        .STRB_WIDTH(32),
        .USE_4_NFC (0),
        .REM_WIDTH (5)
     )

     frame_gen_ll_to_axi_data_i
     (
      // LocalLink input Interface
      .LL_IP_DATA(tx_d_i),
      .LL_IP_SOF_N(tx_sof_n_i),
      .LL_IP_EOF_N(tx_eof_n_i),
      .LL_IP_REM(tx_rem_i),
      .LL_IP_SRC_RDY_N(tx_src_rdy_n_i),
      .LL_OP_DST_RDY_N(tx_dst_rdy_n_i),

      // AXI4-S output signals
      .AXI4_S_OP_TVALID(tx_tvalid_i),
      .AXI4_S_OP_TDATA(tx_tdata_i),
      .AXI4_S_OP_TKEEP(tx_tkeep_i),
      .AXI4_S_OP_TLAST(tx_tlast_i),
      .AXI4_S_IP_TREADY(tx_tready_i),

      .USER_CLK(user_clk_i),
      .RESET(reset2FrameGenCheck)
     );





    // Frame Generator to provide with input to TX_Stream
aurora_64b66b_0_FRAME_GEN frame_gen_i
    (
         .TX_D(tx_d_i),
         .TX_SOF_N(tx_sof_n_i),
         .TX_EOF_N(tx_eof_n_i),
         .TX_REM(tx_rem_i),
         .TX_SRC_RDY_N(tx_src_rdy_n_i),
         .TX_DST_RDY_N(tx_dst_rdy_n_i),




         .CHANNEL_UP(channel_up_i),
         .USER_CLK(user_clk_i),
         .RESET(reset2FrameGenCheck)
    );

     //_____________________________ RX AXI SHIM _______________________________
aurora_64b66b_0_EXAMPLE_AXI_TO_LL #
     (
        .DATA_WIDTH(256),
        .STRB_WIDTH(32),
        .ISUFC(0),
        .REM_WIDTH (5)
     )
     frame_chk_axi_to_ll_data_i
     (
      // AXI4-S input signals
      .AXI4_S_IP_TX_TVALID(rx_tvalid_i),
      .AXI4_S_IP_TX_TREADY(),
      .AXI4_S_IP_TX_TDATA(rx_tdata_i),
      .AXI4_S_IP_TX_TKEEP(rx_tkeep_i),
      .AXI4_S_IP_TX_TLAST(rx_tlast_i),

      // LocalLink output Interface
      .LL_OP_DATA(rx_d_i),
      .LL_OP_SOF_N(rx_sof_n_i),
      .LL_OP_EOF_N(rx_eof_n_i) ,
      .LL_OP_REM(rx_rem_i) ,
      .LL_OP_SRC_RDY_N(rx_src_rdy_n_i),
      .LL_IP_DST_RDY_N(1'b0),

      // System Interface
      .USER_CLK(user_clk_i),
      .RESET(reset2FrameGenCheck),
      .CHANNEL_UP(channel_up_i)
      );



    // Frame Checker to check output from RX_Stream
aurora_64b66b_0_FRAME_CHECK frame_check_i
    (
         .RX_D(rx_d_i),
         .RX_REM(rx_rem_i),
         .RX_SOF_N(rx_sof_n_i),
         .RX_EOF_N(rx_eof_n_i),
         .RX_SRC_RDY_N(rx_src_rdy_n_i),
         .DATA_ERR_COUNT(data_err_count_o),



         .CHANNEL_UP(channel_up_i),
         .USER_CLK(user_clk_i),
         .RESET(reset2FrameGenCheck)
    );
    
 end //end USE_CORE_TRAFFIC=1 block
 else
 begin: no_traffic
     //define traffic generation modules here
 end //end USE_CORE_TRAFFIC=0 block

endgenerate //End generate for USE_CORE_TRAFFIC

     always @(posedge user_clk_i)
         if (reset2FrameGenCheck)
             rx_tvalid_r <=  `DLY 1'b0;
         else if (rx_tvalid_i)
           rx_tvalid_r <=  `DLY 1'b1;
         else 
           rx_tvalid_r <=  `DLY rx_tvalid_r;

     aurora_64b66b_0_cdc_sync_exdes
   # (
      .c_cdc_type    (1),  // 0 Pulse synchronizer, 1 level synchronizer 2 level synchronizer with ACK 
      .c_flop_input  (0),  // 1 Adds one flop stage to the input prmry_in signal
      .c_reset_state (0),  // 1 Reset needed for sync flops 
      .c_single_bit  (1),  // 1 single bit input.
      .c_mtbf_stages (5)   // Number of sync stages needed
     )   u_cdc_rx_tvalid_r
     (
       .prmry_aclk      (user_clk_i),
       .prmry_rst_n     (1'b1 ),
       .prmry_in        (rx_tvalid_r),
       .prmry_vect_in   ('d0 ),
       .scndry_aclk     (INIT_CLK_i ),
       .scndry_rst_n    (1'b1 ),
       .prmry_ack       ( ),
       .scndry_out      (rx_tvalid_r_i),
       .scndry_vect_out ( )
      );

     always @(posedge user_clk_i)
         if (reset2FrameGenCheck)
             usr_clk_counter <=  `DLY 'd0;
         else if (usr_clk_counter >= USR_CLK_PCOUNT)
           usr_clk_counter <=  `DLY USR_CLK_PCOUNT;
         else 
           usr_clk_counter <=  `DLY usr_clk_counter + 1'b1;

     assign usr_clk_count_done = (usr_clk_counter >= USR_CLK_PCOUNT)? 1'b1:1'b0;

    reg usr_clk_count_done_r;
    reg usr_clk_count_done_r2;     

     always @(posedge user_clk_i)
             usr_clk_count_done_r <=  `DLY usr_clk_count_done;

     always @(posedge user_clk_i)
             usr_clk_count_done_r2 <=  `DLY usr_clk_count_done_r;

     aurora_64b66b_0_cdc_sync_exdes
   # (
      .c_cdc_type    (1),  // 0 Pulse synchronizer, 1 level synchronizer 2 level synchronizer with ACK 
      .c_flop_input  (0),  // 1 Adds one flop stage to the input prmry_in signal
      .c_reset_state (0),  // 1 Reset needed for sync flops 
      .c_single_bit  (1),  // 1 single bit input.
      .c_mtbf_stages (5)   // Number of sync stages needed
     )   u_cdc_usr_clk_count_done
     (
       .prmry_aclk      (user_clk_i),
       .prmry_rst_n     (1'b1 ),
       .prmry_in        (usr_clk_count_done_r2),
       .prmry_vect_in   ('d0 ),
       .scndry_aclk     (INIT_CLK_i ),
       .scndry_rst_n    (1'b1 ),
       .prmry_ack       ( ),
       .scndry_out      (usr_clk_count_done_i),
       .scndry_vect_out ( )
      );

     assign lane_up_vio_usrclk = & lane_up_i;

always @(posedge user_clk_i)
begin
    lane_up_vio_usrclk_r1 <= `DLY  lane_up_vio_usrclk;
end

aurora_64b66b_0_cdc_sync_exdes
   # (
      .c_cdc_type    (1),  // 0 Pulse synchronizer, 1 level synchronizer 2 level synchronizer with ACK 
      .c_flop_input  (0),  // 1 Adds one flop stage to the input prmry_in signal
      .c_reset_state (0),  // 1 Reset needed for sync flops 
      .c_single_bit  (1),  // 1 single bit input.
      .c_mtbf_stages (5)   // Number of sync stages needed
     )   u_cdc_lane_up_vio
     (
       .prmry_aclk      (user_clk_i),
       .prmry_rst_n     (1'b1 ),
       .prmry_in        (lane_up_vio_usrclk_r1),
       .prmry_vect_in   ('d0 ),
       .scndry_aclk     (INIT_CLK_i ),
       .scndry_rst_n    (1'b1 ),
       .prmry_ack       ( ),
       .scndry_out      (lane_up_vio_i),
       .scndry_vect_out ( )
      );

     always @(posedge user_clk_i)
           data_err_count_o_r1 <= `DLY |data_err_count_o;

aurora_64b66b_0_cdc_sync_exdes
   # (
      .c_cdc_type    (1),  // 0 Pulse synchronizer, 1 level synchronizer 2 level synchronizer with ACK
      .c_flop_input  (0),  // 1 Adds one flop stage to the input prmry_in signal
      .c_reset_state (0),  // 1 Reset needed for sync flops
      .c_single_bit  (1),  // 1 single bit input.
      .c_mtbf_stages (5)   // Number of sync stages needed
     )   u_cdc_data_err_count_o
     (
       .prmry_aclk      (user_clk_i),
       .prmry_rst_n     (1'b1 ),
       .prmry_in        (data_err_count_o_r1),
       .prmry_vect_in   ('d0 ),
       .scndry_aclk     (INIT_CLK_i ),
       .scndry_rst_n    (1'b1 ),
       .prmry_ack       ( ),
       .scndry_out      (data_err_init_clk_i),
       .scndry_vect_out ( )
      );

reg data_err_count_q;

     always @(posedge user_clk_i)
           data_err_count_q <=  |DATA_ERR_COUNT;

 //---------------------------------------------------------------
 //  VIO Signal Map
 //---------------------------------------------------------------


 (* mark_debug = "true" *)  wire lnk_reset_in_initclk;
 (* mark_debug = "true" *)  wire channel_up_in_initclk;
 (* mark_debug = "true" *)  wire soft_err_in_initclk;
 (* mark_debug = "true" *)  wire data_err_in_initclk;

reg lnk_reset_in_initclk_r;
reg channel_up_in_initclk_r;
reg soft_err_in_initclk_r;

aurora_64b66b_0_cdc_sync_exdes
   # (
      .c_cdc_type    (1),  // 0 Pulse synchronizer, 1 level synchronizer 2 level synchronizer with ACK
      .c_flop_input  (0),  // 1 Adds one flop stage to the input prmry_in signal
      .c_reset_state (0),  // 1 Reset needed for sync flops
      .c_single_bit  (1),  // 1 single bit input.
      .c_mtbf_stages (5)   // Number of sync stages needed
     )   u_cdc_link_reset_i
     (
       .prmry_aclk      (1'b0),
       .prmry_rst_n     (1'b1 ),
       .prmry_in        (link_reset_i),
       .prmry_vect_in   ('d0 ),
       .scndry_aclk     (INIT_CLK_i ),
       .scndry_rst_n    (1'b1 ),
       .prmry_ack       ( ),
       .scndry_out      (lnk_reset_in_initclk),
       .scndry_vect_out ( )
      );

aurora_64b66b_0_cdc_sync_exdes
   # (
      .c_cdc_type    (1),  // 0 Pulse synchronizer, 1 level synchronizer 2 level synchronizer with ACK
      .c_flop_input  (0),  // 1 Adds one flop stage to the input prmry_in signal
      .c_reset_state (0),  // 1 Reset needed for sync flops
      .c_single_bit  (1),  // 1 single bit input.
      .c_mtbf_stages (5)   // Number of sync stages needed
     )   u_cdc_channel_up_in_initclk
     (
       .prmry_aclk      (1'b0),
       .prmry_rst_n     (1'b1 ),
       .prmry_in        (CHANNEL_UP),
       .prmry_vect_in   ('d0 ),
       .scndry_aclk     (INIT_CLK_i ),
       .scndry_rst_n    (1'b1 ),
       .prmry_ack       ( ),
       .scndry_out      (channel_up_in_initclk),
       .scndry_vect_out ( )
      );

aurora_64b66b_0_cdc_sync_exdes
   # (
      .c_cdc_type    (1),  // 0 Pulse synchronizer, 1 level synchronizer 2 level synchronizer with ACK
      .c_flop_input  (0),  // 1 Adds one flop stage to the input prmry_in signal
      .c_reset_state (0),  // 1 Reset needed for sync flops
      .c_single_bit  (1),  // 1 single bit input.
      .c_mtbf_stages (5)   // Number of sync stages needed
     )   u_cdc_soft_err
     (
       .prmry_aclk      (1'b0),
       .prmry_rst_n     (1'b1 ),
       .prmry_in        (SOFT_ERR),
       .prmry_vect_in   ('d0 ),
       .scndry_aclk     (INIT_CLK_i ),
       .scndry_rst_n    (1'b1 ),
       .prmry_ack       ( ),
       .scndry_out      (soft_err_in_initclk),
       .scndry_vect_out ( )
      );

aurora_64b66b_0_cdc_sync_exdes
   # (
      .c_cdc_type    (1),  // 0 Pulse synchronizer, 1 level synchronizer 2 level synchronizer with ACK
      .c_flop_input  (0),  // 1 Adds one flop stage to the input prmry_in signal
      .c_reset_state (0),  // 1 Reset needed for sync flops
      .c_single_bit  (1),  // 1 single bit input.
      .c_mtbf_stages (5)   // Number of sync stages needed
     )   u_cdc_data_err_count_q
     (
       .prmry_aclk      (1'b0),
       .prmry_rst_n     (1'b1 ),
       .prmry_in        (data_err_count_q),
       .prmry_vect_in   ('d0 ),
       .scndry_aclk     (INIT_CLK_i ),
       .scndry_rst_n    (1'b1 ),
       .prmry_ack       ( ),
       .scndry_out      (data_err_in_initclk),
       .scndry_vect_out ( )
      );

     always @(posedge INIT_CLK_i)
     begin
         lnk_reset_in_initclk_r         <=  lnk_reset_in_initclk;
         channel_up_in_initclk_r        <=  channel_up_in_initclk;
         soft_err_in_initclk_r          <=  soft_err_in_initclk;
     end

 (* mark_debug = "true" *)  reg test_passed_r;
 (* mark_debug = "true" *)  reg test_failed_r;
wire reset_quality_cntrs;
wire reset_test_fsm_from_vio;
wire reset_test_enable_from_vio;
wire [2:0] iteraion_cnt_sel_from_vio;
wire dummy_vio_i_3_1;
wire dummy_vio_i_3_2;
wire dummy_vio_i_3_3;
wire [2:0] reset_test_fsm_chk_time_sel;

    (* KEEP = "TRUE" *) reg [3:0] err_cnt_4bit_vio_i;
    (* KEEP = "TRUE" *) reg [1:0] softerr_cnt_2bit_vio_i;
    (* KEEP = "TRUE" *) reg [1:0] dataerr_cnt_2bit_vio_i;
    (* mark_debug = "TRUE" *) (* KEEP = "TRUE" *) reg [19:0] soft_err_cnt_20bit_vio_i;
    (* mark_debug = "TRUE" *) (* KEEP = "TRUE" *) reg [19:0] data_err_cnt_20bit_vio_i;
    (* mark_debug = "TRUE" *) (* KEEP = "TRUE" *) reg [19:0] lnkrst_cnt_20bit_vio_i;
    (* KEEP = "TRUE" *) reg [3:0] chan_up_transcnt_4bit_vio_i;
    (* mark_debug = "TRUE" *) (* KEEP = "TRUE" *) reg [19:0] chan_up_transcnt_20bit_i;
    (* KEEP = "TRUE" *) wire [7:0] vio_probe_in2;

    always @(posedge INIT_CLK_i)
    begin
        if(reset_quality_cntrs ) begin
            soft_err_cnt_20bit_vio_i <= 20'h0;
            chan_up_transcnt_20bit_i  <= 20'h0;
            lnkrst_cnt_20bit_vio_i     <= 20'h0;
            data_err_cnt_20bit_vio_i     <= 20'h0;
        end else begin
            lnkrst_cnt_20bit_vio_i    <= ((~lnk_reset_in_initclk_r & lnk_reset_in_initclk) & (lnkrst_cnt_20bit_vio_i != 20'hfffff))     ?  (lnkrst_cnt_20bit_vio_i + 20'h1)  : lnkrst_cnt_20bit_vio_i;
            chan_up_transcnt_20bit_i  <= ((~channel_up_in_initclk_r & channel_up_in_initclk) & (chan_up_transcnt_20bit_i != 20'hfffff)) ? (chan_up_transcnt_20bit_i + 20'h1) : chan_up_transcnt_20bit_i;
            soft_err_cnt_20bit_vio_i  <= (soft_err_in_initclk & (soft_err_cnt_20bit_vio_i != 20'hfffff)) ? soft_err_cnt_20bit_vio_i + 20'h1 : soft_err_cnt_20bit_vio_i;
            data_err_cnt_20bit_vio_i  <= (data_err_in_initclk & (data_err_cnt_20bit_vio_i != 20'hfffff)) ? data_err_cnt_20bit_vio_i + 20'h1 : data_err_cnt_20bit_vio_i;
        end
    end
    always @(posedge INIT_CLK_i)
    begin
        if(sysreset_from_vio_r3_initclkdomain | gtreset_from_vio_r3 | pma_init_from_fsm_r1 | reset_quality_cntrs ) begin
            err_cnt_4bit_vio_i <= 4'h0;
            softerr_cnt_2bit_vio_i <= 2'h0;
            dataerr_cnt_2bit_vio_i <= 2'h0;
            chan_up_transcnt_4bit_vio_i  <= 4'h0;
        end else begin
            chan_up_transcnt_4bit_vio_i <= ((~channel_up_in_initclk_r & channel_up_in_initclk) & (chan_up_transcnt_4bit_vio_i != 4'hf)) ? (chan_up_transcnt_4bit_vio_i + 4'h1) : chan_up_transcnt_4bit_vio_i;
            err_cnt_4bit_vio_i     <= ((soft_err_in_initclk | (data_err_init_clk_i)) & (err_cnt_4bit_vio_i != 4'hf)) ? err_cnt_4bit_vio_i + 4'h1 : err_cnt_4bit_vio_i;
            softerr_cnt_2bit_vio_i     <= ((soft_err_in_initclk) & (softerr_cnt_2bit_vio_i != 2'h3)) ? softerr_cnt_2bit_vio_i + 2'h1 : softerr_cnt_2bit_vio_i;
            dataerr_cnt_2bit_vio_i     <= ((channel_up_in_initclk) & (usr_clk_count_done_i) & (!rx_tvalid_r_i) & (dataerr_cnt_2bit_vio_i != 2'h3)) ? dataerr_cnt_2bit_vio_i + 2'h1: ((data_err_init_clk_i) & (dataerr_cnt_2bit_vio_i != 2'h3)) ? dataerr_cnt_2bit_vio_i + 2'h1 : dataerr_cnt_2bit_vio_i;
        end
    end

    //assign vio_probe_in2 = {err_cnt_4bit_vio_i, chan_up_transcnt_4bit_vio_i};
    assign vio_probe_in2 = {dataerr_cnt_2bit_vio_i, softerr_cnt_2bit_vio_i, chan_up_transcnt_4bit_vio_i};
//------------------------------------------------------------------------------
     generate
     if(USE_LABTOOLS)
     begin
     
       // Debug UART Control register bit assignments
//       assign sysreset_from_vio_i          = dbg_reg0_r[47];  //1'b0      
//       assign gtreset_from_vio_i           = dbg_reg0_r[45];  //1'b0
//       assign reset_quality_cntrs          = dbg_reg0_r[45];  //1'b0
//       assign reset_test_fsm_from_vio      = dbg_reg0_r[44];  //1'b0
//       assign reset_test_enable_from_vio   = dbg_reg0_r[43];  //1'b0
//       assign reset_test_fsm_chk_time_sel  = dbg_reg0_r[7:6]; //2'h0
//       assign iteraion_cnt_sel_from_vio    = dbg_reg0_r[5:4]; //2'h0
//       assign rx_cdrovrden_i               = dbg_reg0_r[3];   //1'b0
//       assign loopback_i                   = dbg_reg0_r[2:0]; //3'h0
                                
     

//        vio_7series  vio1_inst (
//          .clk                  (INIT_CLK_i),                 // input CLK
//          .probe_in0            (lane_up_vio_i),        // input [0 : 0] PROBE_IN0
//          .probe_in1            (channel_up_in_initclk),         // input [0 : 0] PROBE_IN1
//          .probe_in2            (vio_probe_in2),     // input [7 : 0] PROBE_IN2
//          .probe_out0           (sysreset_from_vio_i), // output [0 : 0] PROBE_OUT0
//          .probe_out1           (gtreset_from_vio_i),  // output [0 : 0] PROBE_OUT1
//          .probe_out2           (rx_cdrovrden_i),      // output [0 : 0] PROBE_OUT2
//          //.probe_out2(link_reset_from_vio_i),      // output [0 : 0] PROBE_OUT2
//          .probe_out3           (loopback_i)           // output [2 : 0] PROBE_OUT3
//        );


//        vio_7series  vio2_inst (
//          .clk(INIT_CLK_i),                 // input CLK
//          .probe_in0(lnk_reset_in_initclk),        // input [0 : 0] PROBE_IN0
//          .probe_in1(soft_err_in_initclk),         // input [0 : 0] PROBE_IN1
//          //.probe_in2(data_err_count_o),     // input [7 : 0] PROBE_IN2
//          .probe_in2(chan_up_transcnt_20bit_i[15:8]),     // input [7 : 0] PROBE_IN2
//          .probe_out0(reset_quality_cntrs), // output [0 : 0] PROBE_OUT0
//          .probe_out1(reset_test_fsm_from_vio),  // output [0 : 0] PROBE_OUT1
//          .probe_out2(reset_test_enable_from_vio),      // output [0 : 0] PROBE_OUT2
//          .probe_out3(iteraion_cnt_sel_from_vio)           // output [2 : 0] PROBE_OUT3
//        );


//        vio_7series  vio3_inst (
//          .clk(INIT_CLK_i),                 // input CLK
//          .probe_in0(test_passed_r),        // input [0 : 0] PROBE_IN0
//          .probe_in1(test_failed_r),         // input [0 : 0] PROBE_IN1
//          .probe_in2(lnkrst_cnt_20bit_vio_i[15:8]),     // input [7 : 0] PROBE_IN2
//          .probe_out0(dummy_vio_i_3_1), // output [0 : 0] PROBE_OUT0
//          .probe_out1(dummy_vio_i_3_2),  // output [0 : 0] PROBE_OUT1
//          .probe_out2(dummy_vio_i_3_3),      // output [0 : 0] PROBE_OUT2
//          .probe_out3(reset_test_fsm_chk_time_sel)           // output [2 : 0] PROBE_OUT3
//        );


        //-----------------------------------------------------------------
        //  ILA core instance
        //-----------------------------------------------------------------
//          ila_7series i_ila (
//            .clk(user_clk_i), // input CLK
//        //  .probe0({tx_d_i[0:15],rx_d_i[0:15],data_err_count_o,lane_up_vio_i,channel_up_i,soft_err_i,hard_err_i,pll_not_locked_i,gt_pll_lock_i,fsm_resetdone_i,17'd0}) // input [63 : 0] PROBE0
//            .probe0({tx_d_i[0:15],rx_d_i[0:15],data_err_count_o,lane_up_vio_usrclk,channel_up_i,soft_err_i,hard_err_i,20'd0}) // input [63 : 0] PROBE0   final
//          );
     //------------------------------------------------------------------------------
     //------------------------------------------------------------------------------
     
//// My ILA     
//  ila_0 (
//    .clk(user_clk_i), // input wire clk
//    .probe0({tx_d_i[0:15],rx_d_i[0:15],data_err_count_o,lane_up_vio_usrclk,channel_up_i,soft_err_i,hard_err_i,20'd0}) // input [63 : 0] PROBE0   final
//  );
     
     
//FSM to do the test
 (* mark_debug = "true" *)  reg [19:0] iteration_cnt;
 (* mark_debug = "true" *)  reg [27:0] timer_r;
 (* mark_debug = "true" *)  reg [2:0] test_fsm_r;

   localparam [2:0]
               IDLE        = 3'h0  ,
               ASSERT_RST  = 3'h1  ,
               DASSERT_RST = 3'h2  ,
               WAIT        = 3'h3  ,
               WAIT1       = 3'h4  ,
               CHECK       = 3'h5  ,
               FAIL        = 3'h6  ,
               DONE        = 3'h7 ;
 (* mark_debug = "true" *)  wire all_checks_ok;
 (* mark_debug = "true" *)  wire all_checks_ok_for_debug;

    assign all_checks_ok = ((chan_up_transcnt_4bit_vio_i == 4'h1) &
                             channel_up_in_initclk                &
                            (soft_err_cnt_20bit_vio_i == 20'h0)   &
                            (data_err_cnt_20bit_vio_i == 20'h0)     );

    assign all_checks_ok_for_debug = (all_checks_ok & (test_fsm_r == CHECK));

 (* mark_debug = "true" *)  wire assert_rst_state_for_debug = (test_fsm_r == ASSERT_RST);
 (* mark_debug = "true" *)  wire dassert_rst_state_for_debug = (test_fsm_r == DASSERT_RST);
 (* mark_debug = "true" *)  wire wait_rst_state_for_debug = (test_fsm_r == WAIT);
 (* mark_debug = "true" *)  wire check_rst_state_for_debug = (test_fsm_r == CHECK);

always @(posedge INIT_CLK_i) begin
    if (reset_test_fsm_from_vio | ~reset_test_enable_from_vio) begin
        test_fsm_r <= IDLE;
        timer_r     <= 28'hFFFFFF;
        pma_init_from_fsm <= 1'b0;
        test_passed_r <= 1'b0;
        test_failed_r <= 1'b0;
        iteration_cnt <= (iteraion_cnt_sel_from_vio == 3'h0) ? 20'd4000 :
                         (iteraion_cnt_sel_from_vio == 3'h1) ? 20'd10000 :
                         (iteraion_cnt_sel_from_vio == 3'h2) ? 20'd20000 :
                         (iteraion_cnt_sel_from_vio == 3'h3) ? 20'd30000 :
                         (iteraion_cnt_sel_from_vio == 3'h4) ? 20'd40000 :
                         (iteraion_cnt_sel_from_vio == 3'h5) ? 20'd50000 :
                         (iteraion_cnt_sel_from_vio == 3'h6) ? 20'd60000 : 20'd100000 ;
    end else begin
        case(test_fsm_r)
        IDLE: begin
            test_passed_r <= 1'b0;
            test_failed_r <= 1'b0;
            pma_init_from_fsm <= 1'b0;
            timer_r <= 28'hFFFFFF;
            if(reset_test_enable_from_vio) begin
                test_fsm_r <= ASSERT_RST;
            end
        end
        ASSERT_RST:begin
            pma_init_from_fsm <= 1'b1;
            timer_r <= timer_r - 28'h1;
            if(timer_r == 28'h0) begin
                timer_r <= 28'h0;
                test_fsm_r <= DASSERT_RST;
            end
        end
        DASSERT_RST: begin
            pma_init_from_fsm <= 1'b0;
            timer_r <= (reset_test_fsm_chk_time_sel == 3'h0) ? 28'hFFFFFFF:
                       (reset_test_fsm_chk_time_sel == 3'h1) ? 28'h7FFFFFF:
                       (reset_test_fsm_chk_time_sel == 3'h2) ? 28'h3FFFFFF:
                       (reset_test_fsm_chk_time_sel == 3'h3) ? 28'h1FFFFFF:
                       (reset_test_fsm_chk_time_sel == 3'h4) ? 28'hFFFFFF:
                       (reset_test_fsm_chk_time_sel == 3'h5) ? 28'h7FFFFF:
                       (reset_test_fsm_chk_time_sel == 3'h6) ? 28'h3FFFFF: 28'h2FFFFF;
            test_fsm_r <= WAIT;
        end
        WAIT: begin
            timer_r <= timer_r - 28'h1;
            if(timer_r == 28'h0) begin
                timer_r <= (reset_test_fsm_chk_time_sel == 3'h0) ? 28'hFFFFFFF:
                           (reset_test_fsm_chk_time_sel == 3'h1) ? 28'h7FFFFFF:
                           (reset_test_fsm_chk_time_sel == 3'h2) ? 28'h3FFFFFF:
                           (reset_test_fsm_chk_time_sel == 3'h3) ? 28'h2FFFFFF:
                           (reset_test_fsm_chk_time_sel == 3'h4) ? 28'h1FFFFFF:
                           (reset_test_fsm_chk_time_sel == 3'h5) ? 28'h7FFFFF:
                           (reset_test_fsm_chk_time_sel == 3'h6) ? 28'h3FFFFF: 28'h2FFFFF;
                test_fsm_r <= (all_checks_ok ? WAIT1 : FAIL);
            end
        end
        WAIT1: begin
            timer_r <= timer_r - 28'h1;
            if(timer_r == 28'h0) begin
                test_fsm_r <= CHECK;
            end
        end
        CHECK: begin
            if(all_checks_ok) begin
                test_fsm_r <= (iteration_cnt == 20'h0) ? DONE : IDLE;
                iteration_cnt  <= iteration_cnt - 20'h1;
            end else begin
                test_fsm_r <= FAIL;
            end
        end
        FAIL: begin
            test_failed_r <= 1'b1;
        end
        DONE: begin
            test_passed_r <= 1'b1;
        end
        endcase
    end
end

     end
     endgenerate
//------------------------------------------------------------------------------
 endmodule
